#include <libgimp/gimp.h>
#include <libgimp/gimpui.h>
#include <gtk/gtk.h>
#include <stdlib.h>

#include "interface.c"


void set_structure_draw_values()
{
  gint  x1, y1, x2, y2;
  gimp_drawable_mask_bounds (drawable->drawable_id, &x1, &y1, &x2, &y2);
  draw_values.x1 = x1;
  draw_values.x2 = x2;
  draw_values.y1 = y1;
  draw_values.y2 = y2;
  draw_values.width = x2 - x1;
  draw_values.height = y2 - y1;
  draw_values.channels = gimp_drawable_bpp (drawable->drawable_id); 
}

void restore_data()
{
  guint8 size = matrix_trans.size;
  gchar c[10];
  for (guint8 i = 0; i < size; i++)
  {
    for (guint8 j = 0; j < size; j++)
    {
      matrix_trans.matrix[i][j] = data_to_save.matrix_trans[i][j]; 
      matrix_cond.matrix[i][j]  = data_to_save.matrix_cond[i][j]; 
    }
  }

  matrix_trans.divider = data_to_save.divider_trans;
  matrix_trans.shift   = data_to_save.shift_trans;

  matrix_cond.divider = data_to_save.divider_cond;
  matrix_cond.shift   = data_to_save.shift_cond;

  mode_values.mode_conditional_left_value  = data_to_save.P1; 
  mode_values.mode_conditional_right_value = data_to_save.P2; 
  mode_values.mode_conditional = data_to_save.conditonal_range;

  mode_values.mode_advanced_left_value  = data_to_save.Q1; 
  mode_values.mode_advanced_right_value = data_to_save.Q2; 
  mode_values.mode_advanced = data_to_save.advanced_range;

  mode_values.mode_variance_sigma = data_to_save.sigma;
  mode_values.mode_variance = data_to_save.variance_range;
}

// no from stucture
void store_data()
{
  // Save data
  guint8 size = matrix_trans.size;
  gchar c[10];
  for (guint8 i = 0; i < size; i++)
  {
    for (guint8 j = 0; j < size; j++)
    {
      data_to_save.matrix_trans[i][j] = matrix_trans.matrix[i][j];
      data_to_save.matrix_cond[i][j] = matrix_cond.matrix[i][j];
    }
  }

  data_to_save.divider_trans = matrix_trans.divider;
  data_to_save.shift_trans = matrix_trans.shift;

  data_to_save.divider_cond = matrix_cond.divider;
  data_to_save.shift_cond = matrix_cond.shift;

  data_to_save.P1 = mode_values.mode_conditional_left_value;
  data_to_save.P2 = mode_values.mode_conditional_right_value;
  data_to_save.conditonal_range = mode_values.mode_conditional; 

  data_to_save.Q1 = mode_values.mode_advanced_left_value;
  data_to_save.Q2 = mode_values.mode_advanced_right_value;
  data_to_save.advanced_range = mode_values.mode_advanced;

  data_to_save.sigma = mode_values.mode_variance_sigma;
  data_to_save.variance_range = mode_values.mode_variance;
}

// Forward declarations 
static void  query (void);
static void  run (const gchar      *name,
                        gint       nparams,                  
		  const GimpParam  *param,
			gint       *nreturn_vals,
			GimpParam  **return_vals);

const GimpPlugInInfo PLUG_IN_INFO =
{
  NULL,    // init_proc
  NULL,    // quit_proc
  query,   // query_proc
  run,     // run_proc
};

static char str[100];

MAIN()

// Query plug-in 
static void query (void)
{
	static GimpParamDef args[] = {
		{ GIMP_PDB_INT32, "run-mode", "Run mode" },
		{ GIMP_PDB_IMAGE, "image", "Input image" },
		{ GIMP_PDB_DRAWABLE, "drawable", "Input drawable" }
	};

	gimp_install_procedure (CONDITIONAL_CONVOLUTION_MATRIX_PROC,
   												"Conditional Convolution Matrix",
													"Image Modification by mask", 
							 						"Daniel Zmudziński",
												  "Daniel Zmudziński",
											 	  "2016",
												  "_Warunkowy filtr macierzowy...",
											 	  "RGB*, GRAY*",
											 	  GIMP_PLUGIN,
											 	  G_N_ELEMENTS (args), 0,
											 	  args, NULL);

	gimp_plugin_menu_register (CONDITIONAL_CONVOLUTION_MATRIX_PROC,
	                           "<Image>/Filters/Misc"); 
}

// run plug-in 
static void run (const gchar      *name,
								 gint              nparams,
								 const GimpParam  *param,
								 gint             *nreturn_vals,
								 GimpParam        **return_vals)
{
	static GimpParam  values[2];
	GimpPDBStatusType status = GIMP_PDB_SUCCESS;
	GimpRunMode       run_mode;

	// Setting mandatory output values 
	*nreturn_vals = 1;
	*return_vals  = values;

	values[0].type = GIMP_PDB_STATUS;
	values[0].data.d_status = status;

  // restore settings saved in GIMP core 
  gimp_get_data (CONDITIONAL_CONVOLUTION_MATRIX_PROC, &data_to_save);
  restore_data();


	image_ID = param[1].data.d_image;
  drawable = gimp_drawable_get (param[2].data.d_drawable);

  // Set values in structure
  set_structure_draw_values();

	// Getting run_mode - we won't display a dialog if 
	// we are in NONINTERACTIVE mode 
	run_mode = param[0].data.d_int32;

	if (run_mode == GIMP_RUN_INTERACTIVE)
  {
	  if (!user_interface ("Warunkowy filtr macierzowy", 
        CONDITIONAL_CONVOLUTION_MATRIX_PROC, drawable)) 
    {
      gimp_drawable_detach (drawable);
      status = GIMP_PDB_EXECUTION_ERROR;
      //store_data();  
      return;
    }
  }

  sprintf(str, "BLA BLA");
  gimp_progress_init (str);

  gimp_drawable_flush (drawable);
  gimp_drawable_merge_shadow (drawable->drawable_id, TRUE);
  gimp_drawable_update (drawable->drawable_id, 
      draw_values.x1, draw_values.y1, draw_values.width, draw_values.height);

 // flag_save_changes = TRUE;
  //convolve_image();

  gimp_displays_flush ();
  gimp_drawable_detach (drawable);

  // save settings in the GIMP core 
  store_data();  
  gimp_set_data (CONDITIONAL_CONVOLUTION_MATRIX_PROC, &data_to_save,
		sizeof (DataToSave));
}
