#include <glib.h>

#include "utils.c"
static char str[100];

void convolve_image_cal (GimpDrawable *drawable, GimpPreview *preview,
  void (* process_row) (guchar **, guchar *));

// main functions
static void free_mem (guchar ***row, guchar **outrow);
static void init_mem (guchar ***row, guchar **outrow);
static void shuffle (GimpPixelRgn *rgn_in, guchar **row, gint ypos); 
static void set_border(GimpPixelRgn rgn_in, GimpPixelRgn rgn_out);

// process pixel
static void process_pixel_basic                 
            (guchar **row, guchar *outrow);
static void process_pixel_conditional_in_range  
            (guchar **row, guchar *outrow);
static void process_pixel_conditional_out_range 
            (guchar **row, guchar *outrow);

// Advanced
static void process_pixel_advanced_in_range_matrix_cond 
            (guchar **row, guchar *outrow);
static void process_pixel_advanced_in_range_no_matrix_cond 
            (guchar **row, guchar *outrow);
static void process_pixel_advanced_out_range_matrix_cond 
            (guchar **row, guchar *outrow);
static void process_pixel_advanced_out_range_no_matrix_cond 
            (guchar **row, guchar *outrow);

// Variance
static void process_pixel_variance_in_range_matrix_cond 
            (guchar **row, guchar *outrow);
static void process_pixel_variance_in_range_no_matrix_cond 
            (guchar **row, guchar *outrow);
static void process_pixel_variance_out_range_matrix_cond 
            (guchar **row, guchar *outrow);
static void process_pixel_variance_out_range_no_matrix_cond 
            (guchar **row, guchar *outrow);

// Helpful functions
static gfloat clamp_8bit_range(gfloat number);
static gfloat average_weighted_matrix_cond (guchar **row);
static gfloat average_weighted_matrix_trans (guchar **row);
static gfloat variance_matrix_cond 
              (guchar **row, gfloat average_weighted);
static gfloat variance_matrix_trans 
              (guchar **row, gfloat average_weighted);



void convolve_image_cal(GimpDrawable *drawable, GimpPreview *preview, 
    void (* process_pixel) (guchar **, guchar *))
{
  gint         i, j, k;// ii; 
  GimpPixelRgn rgn_in, rgn_out;
  guchar       **row;
  guchar       *outrow;

  guchar       **row_ptr;
  guchar       *outrow_ptr;


  gimp_tile_cache_ntiles (2 * (draw_values.width / gimp_tile_width () + 1));
  // Initialises two PixelRgns, one to read original data,
  // and the other to write output data. That second one will
  // be merged at the end by the call to gimp_drawable_merge_shadow() 
  gimp_pixel_rgn_init (&rgn_in , drawable, draw_values.x1, draw_values.y1, 
      draw_values.width, draw_values.height, FALSE, FALSE);
  gimp_pixel_rgn_init (&rgn_out, drawable, draw_values.x1, draw_values.y1, 
      draw_values.width, draw_values.height, TRUE , TRUE);

  // Set x, y (start, end) 
  gint xs = draw_values.x1 + matrix_trans.radius; 
  gint ys = draw_values.y1 + matrix_trans.radius;
  gint xe = draw_values.x2 - matrix_trans.radius; 
  gint ye = draw_values.y2 - matrix_trans.radius; 

  // Initialise enough memory for rows
  init_mem(&row, &outrow);  

  // Get rows
  for (i = 0; i < matrix_trans.size; i++)
  {
    gimp_pixel_rgn_get_row(&rgn_in, row[i], draw_values.x1, 
        draw_values.y1 + i, draw_values.width);
  }

  row_ptr = g_new (guchar *, matrix_trans.size);


  gimp_progress_init ("START");
  // main function *******************************************************
  for (i = ys; i < ye; i++)
  {
    //start pointer    
    outrow_ptr = outrow + (2 * draw_values.channels);
    for (gint iii = 0; iii < matrix_trans.size; iii++)
      *(row_ptr + iii) = *(row + iii);


    for (j = xs; j < xe ; j++)
    {
      for (k = 0; k < draw_values.channels; k++)
      {
        if (mode_values.channels[k] == 1)
          process_pixel (row_ptr, outrow_ptr);
        else
          *outrow_ptr = 
            (*(*(row_ptr + 2) + (guchar)(draw_values.channels * 2)));

        // shift pointer
        outrow_ptr += 1;
        for (gint iii = 0; iii < matrix_trans.size; iii++)
            *(row_ptr + iii) += 1;
      }
    }
   
    gimp_pixel_rgn_set_row(&rgn_out, outrow, draw_values.x1, i, 
        draw_values.width);

    shuffle(&rgn_in, row, i);
    
    // progress bar
    if (i % 10 == 0) gimp_progress_update ((gdouble)(i - draw_values.y1) / 
        (gdouble)(draw_values.y2 - draw_values.y1));
  }
  gimp_progress_update (1.0);
  gimp_progress_init ("END");
  
  set_border(rgn_in, rgn_out);

  // free memory
  free_mem(&row, &outrow); 
  //  Update the modified region
  gimp_drawable_preview_draw_region (GIMP_DRAWABLE_PREVIEW (preview), &rgn_out);
  draw_region = rgn_out;
}



// process pixel
static void
process_pixel_basic (guchar **row, guchar *outrow)
{
  gfloat result = average_weighted_matrix_trans(row);
  result = clamp_8bit_range(result);
  *outrow = (gchar)result;
}

static void
process_pixel_conditional_in_range (guchar **row, guchar *outrow)
{
  gint act_val = (*(*(row + 2) + (guchar)(draw_values.channels * 2)));
  gfloat result = 0.0;

  if (act_val >= mode_values.mode_conditional_left_value 
      && act_val <= mode_values.mode_conditional_right_value) 
  {
    result = average_weighted_matrix_trans(row);
    result = clamp_8bit_range(result);

  } else result = act_val;

  *outrow = (gchar)result;
}

static void 
process_pixel_conditional_out_range (guchar **row, guchar *outrow)
{
  gint act_val = (*(*(row + 2) + (guchar)(draw_values.channels * 2)));
  gfloat result = 0.0;

  if (act_val < mode_values.mode_conditional_left_value 
      || act_val > mode_values.mode_conditional_right_value) 
  {
    result = average_weighted_matrix_trans(row);
    result = clamp_8bit_range(result);

  } else result = act_val;

  *outrow = (gchar)result;
}


//**************************
// mode advanced functions in range 


static void
process_pixel_advanced_in_range_matrix_cond (guchar **row, guchar *outrow)
{
  gfloat result = 0.0;
  gfloat average_weighted = average_weighted_matrix_cond(row);

  if (average_weighted >= mode_values.mode_advanced_left_value 
      && average_weighted <= mode_values.mode_advanced_right_value) 
  {
    result = average_weighted_matrix_trans(row);
    result = clamp_8bit_range(result);

  } else result = (*(*(row + 2) + (guchar)(draw_values.channels * 2)));
  
  *outrow = (gchar)result;
}

static void
process_pixel_advanced_in_range_no_matrix_cond (guchar **row, guchar *outrow)
{
  gfloat result = average_weighted_matrix_trans(row);
  if (result >= mode_values.mode_advanced_left_value
      && result <= mode_values.mode_advanced_right_value) 
  {
    result = clamp_8bit_range(result);

  } else result = (*(*(row + 2) + (guchar)(draw_values.channels * 2)));

  *outrow = (gchar)result;
}

// mode advanced functions out range 

static void
process_pixel_advanced_out_range_matrix_cond (guchar **row, guchar *outrow)
{
  gfloat result = 0.0;
  gfloat average_weighted = average_weighted_matrix_cond(row);

  if (average_weighted < mode_values.mode_advanced_left_value
      || average_weighted > mode_values.mode_advanced_right_value) 
  {
    result = average_weighted_matrix_trans(row);
    result = clamp_8bit_range(result);

  } else result = (*(*(row + 2) + (guchar)(draw_values.channels * 2)));

  *outrow = (gchar)result;
}

static void
process_pixel_advanced_out_range_no_matrix_cond (guchar **row, guchar *outrow)
{
  gfloat result = average_weighted_matrix_trans(row);
  if (result < mode_values.mode_advanced_left_value 
      || result > mode_values.mode_advanced_right_value) 
  {
    result = clamp_8bit_range(result);

  } else result = (*(*(row + 2) + (guchar)(draw_values.channels * 2)));

  *outrow = (gchar)result;
}

//***************************
// mode variance functions in range 

static void
process_pixel_variance_in_range_matrix_cond (guchar **row, guchar *outrow)
{
  gint act_val = (*(*(row + 2) + (guchar)(draw_values.channels * 2)));
  gfloat result = 0.0;

  gfloat average_weighted = average_weighted_matrix_cond(row);

  gfloat variance = variance_matrix_cond(row, average_weighted);
  variance = abs(variance);
  gfloat standard_deviation =  sqrt((gdouble)variance);

  gfloat distance  = standard_deviation * mode_values.mode_variance_sigma;
  distance         = round(distance);
  gint left_value  = average_weighted - (gint)distance;
  gint right_value = average_weighted + (gint)distance;

  if (act_val >= left_value && act_val <= right_value) 
  {
    result = average_weighted_matrix_trans(row);
    result = clamp_8bit_range(result);

  } else result = act_val;

  *outrow = (gchar)result;
}

static void
process_pixel_variance_in_range_no_matrix_cond (guchar **row, guchar *outrow)
{
  gfloat result = 0.0;
  gint act_val = (*(*(row + 2) + (guchar)(draw_values.channels * 2)));

  gfloat average_weighted = average_weighted_matrix_trans(row);
  result = average_weighted;

  gfloat variance = variance_matrix_trans(row, average_weighted);
  variance = abs(variance);
  gfloat standard_deviation =  sqrt((gdouble)variance);

  gfloat distance    = standard_deviation * mode_values.mode_variance_sigma;
  distance    = round(distance);
  gint left_value  = average_weighted - (gint)distance;
  gint right_value = average_weighted + (gint)distance;

  if (act_val >= left_value && act_val <= right_value) 
  {
    result = clamp_8bit_range(result);

  } else result = act_val;

  *outrow = (gchar)result;
}

// mode variance functions out range 

static void
process_pixel_variance_out_range_matrix_cond (guchar **row, guchar *outrow)
{
  gfloat result = 0.0;
  gint act_val = (*(*(row + 2) + (guchar)(draw_values.channels * 2)));
  gfloat average_weighted = average_weighted_matrix_cond(row);

  gfloat variance = variance_matrix_cond(row, average_weighted);
  variance = abs(variance);
  gfloat standard_deviation =  sqrt((gdouble)variance);

  gfloat distance  = standard_deviation * mode_values.mode_variance_sigma;
  distance         = round(distance);
  gint left_value  = average_weighted - (gint)distance;
  gint right_value = average_weighted + (gint)distance;

  if ((act_val < left_value) || (act_val > right_value)) 
  {
    result = average_weighted_matrix_trans(row);
    result = clamp_8bit_range(result);

  } else result = act_val;

  *outrow = (gchar)result;
}

static void
process_pixel_variance_out_range_no_matrix_cond (guchar **row, guchar *outrow)
{
  gfloat result = 0.0;
  gint act_val = (*(*(row + 2) + (guchar)(draw_values.channels * 2)));

  gfloat average_weighted = average_weighted_matrix_trans(row);
  result = average_weighted; 

  gfloat variance = variance_matrix_trans(row, average_weighted);
  variance = abs(variance);
  gfloat standard_deviation =  sqrt((gdouble)variance);

  gfloat distance  = standard_deviation * mode_values.mode_variance_sigma;
  distance         = round(distance);
  gint left_value  = average_weighted - (gint)distance;
  gint right_value = average_weighted +(gint)distance;


  if ((act_val < left_value) || (act_val > right_value)) 
  {
    result = clamp_8bit_range(result);

  } else result = act_val;

  *outrow = (gchar)result;
}

// end out of range
//
//
// **********************************************
//  PROCESS PIXEL END
//
//
//


// Helpful functions
static gfloat clamp_8bit_range(gfloat number)
{
  if (number > 255.0) number = 255.0;
    else if (number < 0.0) number = 0.0; 
  return round(number);
}

gfloat average_weighted_matrix_cond (guchar **row)
{
  guint x  = 0;
  guint y  = 0;
  gfloat average_weighted = 0.0;

  for (guint i = 0; i < matrix_cond.size_index; i++)
  {
    x = matrix_cond.tab_index[0][i];
    y = matrix_cond.tab_index[1][i];

    average_weighted += (*(*(row + x) + (guchar)(draw_values.channels * y)) 
        * matrix_cond.matrix[x][y]);
  }

  average_weighted /= (gfloat)matrix_cond.divider;
  average_weighted += matrix_cond.shift;

  return average_weighted;
}


gfloat average_weighted_matrix_trans (guchar **row)
{
  guint x  = 0;
  guint y  = 0;
  gfloat average_weighted = 0.0;

  for (guint i = 0; i < matrix_trans.size_index; i++)
  {
    x = matrix_trans.tab_index[0][i];
    y = matrix_trans.tab_index[1][i];

    average_weighted += (*(*(row + x) + (guchar)(draw_values.channels * y)) 
        * matrix_trans.matrix[x][y]);
  }

  average_weighted /= (gfloat)matrix_trans.divider;
  average_weighted += matrix_trans.shift;

  return average_weighted;
}


gfloat variance_matrix_cond (guchar **row, gfloat average_weighted)
{
  guint x = 0;
  guint y = 0;
  gfloat variance = 0.0;
  gfloat tmp = 0.0;

  for (guint i = 0; i < matrix_cond.size_index; i++)
  {
    x = matrix_cond.tab_index[0][i];
    y = matrix_cond.tab_index[1][i];

    tmp = *(*(row + x) + (guchar)(draw_values.channels * y)) - average_weighted;

    variance += matrix_cond.matrix[x][y] * tmp * tmp;
  }

  return variance;
}

gfloat variance_matrix_trans (guchar **row, gfloat average_weighted)
{
  guint x = 0;
  guint y = 0;
  gfloat variance = 0.0;
  gfloat tmp = 0;

  // Variance
  for (guint i = 0; i < matrix_trans.size_index; i++)
  {
    x = matrix_trans.tab_index[0][i];
    y = matrix_trans.tab_index[1][i];

    tmp = *(*(row + x) + (guchar)(draw_values.channels * y)) - average_weighted;

    variance += matrix_trans.matrix[x][y] * tmp * tmp;
  }

  return variance;
}

// ****************************************************************
// main functions

static void
shuffle (GimpPixelRgn *rgn_in, guchar **row, gint ypos)
{
  guchar *tmp_row;

  // Get tile row (i + radius + 1) into row[0] 
  gimp_pixel_rgn_get_row (rgn_in, row[0], draw_values.x1, 
      ypos + matrix_trans.radius + 1, draw_values.width); 

  // Permute row[i] with row[i-1] and row[0] with row[2r] 
  tmp_row = row[0];
  for (gint i = 1; i < matrix_trans.size; i++)
    row[i - 1] = row[i];
  row[2 * matrix_trans.radius] = tmp_row;
}

static void free_mem (guchar ***row, guchar **outrow)
{
  // free memory
  for (gint i = 0; i < matrix_trans.size; i++) g_free((*row)[i]);
  
  g_free (*row);
  g_free (*outrow);
}

static void init_mem (guchar ***row, guchar **outrow)
{
  gint num_bytes = draw_values.width*draw_values.channels;

  // Allocate enough memory for row and outrow 
  *row = g_new (guchar *, matrix_trans.size);

  for (gint i = 0; i < matrix_trans.size; i++)
    (*row)[i] = g_new (guchar, num_bytes);

  *outrow = g_new (guchar, num_bytes);
}


static void set_border(GimpPixelRgn rgn_in, GimpPixelRgn rgn_out)
{
  guchar *row = g_new (guchar, draw_values.width*draw_values.channels);
  guchar *col = g_new (guchar, draw_values.height*draw_values.channels);

  // Set rows
  // set row 0
  gimp_pixel_rgn_get_row(&rgn_in, row, 
      draw_values.x1, draw_values.y1, draw_values.width); 
  gimp_pixel_rgn_set_row(&rgn_out, row, 
      draw_values.x1, draw_values.y1, draw_values.width);

  
  // set row 1
  gimp_pixel_rgn_get_row(&rgn_in, row,
      draw_values.x1, draw_values.y1+1, draw_values.width); 
  gimp_pixel_rgn_set_row(&rgn_out, row, 
      draw_values.x1, draw_values.y1+1, draw_values.width);

  // set row end-1
  gimp_pixel_rgn_get_row(&rgn_in, row, draw_values.x1, 
      draw_values.y2-matrix_trans.radius, draw_values.width); 
  gimp_pixel_rgn_set_row(&rgn_out, row, draw_values.x1, 
      draw_values.y2-matrix_trans.radius, draw_values.width);

  // set row end
  gimp_pixel_rgn_get_row(&rgn_in, row, draw_values.x1, 
      draw_values.y2-matrix_trans.radius+1, draw_values.width); 
  gimp_pixel_rgn_set_row(&rgn_out, row, draw_values.x1, 
      draw_values.y2-matrix_trans.radius+1, draw_values.width);

  // Set columns **********************
  guint height_right = draw_values.height-(2*matrix_trans.radius);

  // set col 0
  gimp_pixel_rgn_get_col(&rgn_in, col, draw_values.x1, 
      draw_values.y1+matrix_trans.radius, height_right); 
  gimp_pixel_rgn_set_col(&rgn_out, col, draw_values.x1, 
      draw_values.y1+matrix_trans.radius, height_right);

  // set col 1
  gimp_pixel_rgn_get_col(&rgn_in, col, draw_values.x1+1, 
      draw_values.y1+matrix_trans.radius, height_right); 
  gimp_pixel_rgn_set_col(&rgn_out, col, draw_values.x1+1, 
      draw_values.y1+matrix_trans.radius, height_right);

  // set col end-1
  gimp_pixel_rgn_get_col(&rgn_in, col, draw_values.x2-matrix_trans.radius, 
      draw_values.y1+matrix_trans.radius, height_right); 
  gimp_pixel_rgn_set_col(&rgn_out, col, draw_values.x2-matrix_trans.radius, 
      draw_values.y1+matrix_trans.radius, height_right);

  // set col end
  gimp_pixel_rgn_get_col(&rgn_in, col, draw_values.x2-matrix_trans.radius+1, 
      draw_values.y1+matrix_trans.radius, height_right); 
  gimp_pixel_rgn_set_col(&rgn_out,col, draw_values.x2-matrix_trans.radius+1, 
      draw_values.y1+matrix_trans.radius, height_right);

  g_free (row);
  g_free (col);
}

