#include "conv.c"
#include <time.h>
#include <string.h>
#include <unistd.h>

#define CONDITIONAL_CONVOLUTION_MATRIX_PROC "plug-in-conditional-convolution-matrix"
#define PLUG_IN_BINARY "conditional-convolution-matrix"
#define PLUG_IN_ROLE   "gimp-conditional-convolution-matrix"
#define PREVIEW_SIZE 480

static gint32       image_ID                = 0;
static GtkWidget    *preview                = NULL;
static GtkWidget    *preview_hbox           = NULL;
static GimpDrawable *drawable               = 0;
static char         str[100];

// Matrix transformation
static GtkWidget    *entry_matrix_trans[5][5];
static GtkWidget    *entry_divisor_trans       = NULL;
static GtkWidget    *entry_shift_trans         = NULL;

// Matrix conditional
static GtkWidget    *entry_matrix_cond[5][5];
static GtkWidget    *check_is_use_matrix_cond  = NULL;
static GtkWidget    *label_matrix_cond         = NULL;
static GtkWidget    *label_divisor_cond        = NULL;
static GtkWidget    *entry_divisor_cond        = NULL;
static GtkWidget    *label_shift_cond          = NULL;
static GtkWidget    *entry_shift_cond          = NULL;

// Checkbox - channels
static GtkWidget    *channel_red               = NULL;
static GtkWidget    *channel_green             = NULL;
static GtkWidget    *channel_blue              = NULL;
static GtkWidget    *channel_grey              = NULL;
static GtkWidget    *channel_alfa              = NULL;

// Radio buttons - mode 
static GtkWidget    *radio_basic            = NULL;
static GtkWidget    *radio_conditional      = NULL;
static GtkWidget    *radio_advanced         = NULL;
static GtkWidget    *radio_variance         = NULL;

// Controls mode conditional
static GtkWidget    *label_P1                    = NULL;
static GtkWidget    *entry_text_P1               = NULL;
static GtkWidget    *label_P2                    = NULL;
static GtkWidget    *entry_text_P2               = NULL;
static GtkWidget    *radio_conditional_in_range  = NULL;
static GtkWidget    *radio_conditional_out_range = NULL;

// Controls mode conditional
static GtkWidget    *label_Q1                    = NULL;
static GtkWidget    *entry_Q1                    = NULL;
static GtkWidget    *label_Q2                    = NULL;
static GtkWidget    *entry_Q2                    = NULL;
static GtkWidget    *radio_advanced_in_range     = NULL;
static GtkWidget    *radio_advanced_out_range    = NULL;

// Controls mode variance
static GtkWidget    *label_sigma                 = NULL;
static GtkWidget    *entry_text_sigma            = NULL;
static GtkWidget    *radio_variance_in_range     = NULL;
static GtkWidget    *radio_variance_out_range    = NULL;

void matrix_conditional_sensitivity()
{
  gboolean sensitivity = gtk_toggle_button_get_active
    ((GtkToggleButton *)check_is_use_matrix_cond);

  gtk_widget_set_sensitive (label_matrix_cond, sensitivity);

  for (guint8 i = 0; i < matrix_trans.size; i++)
  {
    for (guint8 j = 0; j < matrix_trans.size; j++)
    {
      gtk_widget_set_sensitive (entry_matrix_cond[i][j], sensitivity);
    }
  }

  gtk_widget_set_sensitive (label_divisor_cond, sensitivity);
  gtk_widget_set_sensitive (entry_divisor_cond, sensitivity);
  gtk_widget_set_sensitive (label_shift_cond  , sensitivity);
  gtk_widget_set_sensitive (entry_shift_cond  , sensitivity);
}

// if matrix transformation and matrix conditional is similar 
// return 1, when matrixs is different return 0
// + take account divider and shift
gint is_similar_matrix_tans_and_cond()
{
  guint8 size = matrix_trans.size;
    for (guint8 i = 0; i < size; i++)
      {
        for (guint8 j = 0; j < size; j++)
		{
        if (matrix_trans.matrix[i][j] != matrix_cond.matrix[i][j]) return 0;
    }
  }

  if (matrix_trans.divider != matrix_cond.divider) return 0;
  if (matrix_trans.shift != matrix_cond.shift) return 0;

  return 1;
}

void update_matrix_trans()
{
  const gchar *strr;
  gint valuee = 0;
  gint sum = 0;

  for (guint i = 0; i < 5; i++)
  {

    for (guint j = 0; j < 5; j++)
    {
      valuee = get_number((GtkEntry *)entry_matrix_trans[i][j]);
      matrix_trans.matrix[i][j] = valuee;

      if (valuee != 0)
      {
        matrix_trans.tab_index[0][sum] = i; 
        matrix_trans.tab_index[1][sum] = j; 
        sum++;
      }
      
    }
  }
  matrix_trans.size_index = sum;
}

void update_matrix_cond()
{
  const gchar *strr;
  gint valuee = 0;
  gint sum = 0;

  for (guint i = 0; i < 5; i++)
  {
    for (guint j = 0; j < 5; j++)
    {
      valuee = get_number((GtkEntry *)entry_matrix_cond[i][j]);
      matrix_cond.matrix[i][j] = valuee;

      if (valuee != 0)
      {
        matrix_cond.tab_index[0][sum] = i; 
        matrix_cond.tab_index[1][sum] = j; 
        sum++;
      }
      
    }
  }
  matrix_cond.size_index = sum;
}

void update_channels()
{
  gint valuee = 0;
  gint sum = 0;
  guint tab[4];

  if (draw_values.channels < 3) // grey channels
  {
    if (gtk_toggle_button_get_active((GtkToggleButton *)channel_grey) == 1
        && channel_grey != NULL)
      tab[0] = 1; else tab[0] = 0;

    if (gtk_toggle_button_get_active((GtkToggleButton *)channel_alfa) == 1
        && channel_alfa != NULL)
      tab[1] = 1; else tab[1] = 0;
  } else // rgb channels
  {
    if (gtk_toggle_button_get_active((GtkToggleButton *)channel_red) == 1
        && channel_red != NULL)
      tab[0] = 1; else tab[0] = 0;
  
    if (gtk_toggle_button_get_active((GtkToggleButton *)channel_green) == 1
        && channel_green != NULL)
      tab[1] = 1; else tab[1] = 0;

    if (gtk_toggle_button_get_active((GtkToggleButton *)channel_blue) == 1
        && channel_blue != NULL)
      tab[2] = 1; else tab[2] = 0;

    if (gtk_toggle_button_get_active((GtkToggleButton *)channel_alfa) == 1
        && channel_alfa != NULL)
      tab[3] = 1; else tab[3] = 0;
  }

  mode_values.channels[0]    = tab[0];
  mode_values.channels[1]    = tab[1];
  mode_values.channels[2]    = tab[2];
  mode_values.channels[3]    = tab[3];
}

void convolve_image()
{
  //const gchar *strr;
  gint   valuee  = 0;
  gfloat valueef = 0.0;
  gint use_matrix_cond = 0;

  update_channels();

  update_matrix_trans();
  // Divider 
  valuee = get_number((GtkEntry *)entry_divisor_trans);
  matrix_trans.divider = valuee;
  // Shifft
  valuee = get_number((GtkEntry *)entry_shift_trans);
  matrix_trans.shift = valuee;



  // start
  clock_t start = clock(), diff;

  switch (mode_values.mode)
  {
    case 1:
      convolve_image_cal(drawable, (GimpPreview *)preview, 
          process_pixel_basic);
      break;
      
    case 2:        
      valuee = get_number((GtkEntry *)entry_text_P1);
      mode_values.mode_conditional_left_value  = valuee;
      
      valuee = get_number((GtkEntry *)entry_text_P2);
      mode_values.mode_conditional_right_value  = valuee;

      switch (mode_values.mode_conditional)
      {
        case 1:
          convolve_image_cal(drawable, (GimpPreview *)preview, 
              process_pixel_conditional_in_range);
          break;

        case 2:
          convolve_image_cal(drawable, (GimpPreview *)preview, 
              process_pixel_conditional_out_range);
          break;

        default:

          break;
      }
    break;

    case 3:        
      valuee = get_number((GtkEntry *)entry_Q1);
      mode_values.mode_advanced_left_value  = valuee;
      
      valuee = get_number((GtkEntry *)entry_Q2);
      mode_values.mode_advanced_right_value  = valuee;

      update_matrix_cond();
      
      // Divider 
      valuee = get_number((GtkEntry *)entry_divisor_cond);
      matrix_cond.divider = valuee;
      // Shifft
      valuee = get_number((GtkEntry *)entry_shift_cond);
      matrix_cond.shift = valuee;

      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON 
         (check_is_use_matrix_cond))) 
      {
        use_matrix_cond = !is_similar_matrix_tans_and_cond(); 
      } else use_matrix_cond = 0; 

      switch (mode_values.mode_advanced)
      {
        case 1:
          if (use_matrix_cond == 1)
          {
            convolve_image_cal(drawable, (GimpPreview *)preview, 
                process_pixel_advanced_in_range_matrix_cond);
          } else
          {
            convolve_image_cal(drawable, (GimpPreview *)preview, 
                process_pixel_advanced_in_range_no_matrix_cond);
          }
          break;

        case 2:
          if (use_matrix_cond == 1)
          {
            convolve_image_cal(drawable, (GimpPreview *)preview, 
                process_pixel_advanced_out_range_matrix_cond);
          } else
          {
            convolve_image_cal(drawable, (GimpPreview *)preview, 
                process_pixel_advanced_out_range_no_matrix_cond);
          }
          break;

        default:

          break;
      }
    break;

    case 4:
      valueef = get_number_float((GtkEntry *)entry_text_sigma);
      mode_values.mode_variance_sigma  = valueef;

      update_matrix_cond();
      // Divider 
      valuee = get_number((GtkEntry *)entry_divisor_cond);
      matrix_cond.divider = valuee;
      // Shifft
      valuee = get_number((GtkEntry *)entry_shift_cond);
      matrix_cond.shift = valuee;

      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON 
         (check_is_use_matrix_cond))) 
      {
        use_matrix_cond = !is_similar_matrix_tans_and_cond();
      } else use_matrix_cond = 0; 

      switch (mode_values.mode_variance)
      {
        case 1:
          if (use_matrix_cond == 1)
          {
            convolve_image_cal(drawable, (GimpPreview *)preview, 
                process_pixel_variance_in_range_matrix_cond);
          } else
          {
            convolve_image_cal(drawable, (GimpPreview *)preview, 
                process_pixel_variance_in_range_no_matrix_cond);
          }
          break;

        case 2:
          if (use_matrix_cond == 1)
          {
            convolve_image_cal(drawable, (GimpPreview *)preview, 
                process_pixel_variance_out_range_matrix_cond);
          } else
          {
            convolve_image_cal(drawable, (GimpPreview *)preview, 
                process_pixel_variance_out_range_no_matrix_cond);
          }
          break;

        default:

          break;
      }
      break;

    default:
      break;
  
  }
  

  // end
  diff = clock() - start;
  int msec = diff * 1000 / CLOCKS_PER_SEC;
/*
  gint cos = 0;
  switch (mode_values.mode)
  {
    case 2:
      cos = mode_values.mode_conditional;
      break;
    case 3:
      cos = mode_values.mode_advanced;
      break;
    case 4:
      cos = mode_values.mode_variance;
      break;
  }
*/
}

void update_preview()
{
  gimp_drawable_preview_draw_region (GIMP_DRAWABLE_PREVIEW (preview),
					 &draw_region);
}

void update_mode_conditional(GtkWidget *w, gpointer data)
{

  gint active = 0;
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON 
        (radio_conditional_in_range))) 
  { 
    active = 1;
  }
    
    
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON 
        (radio_conditional_out_range))) 
  {
    active = 2;
  }

  mode_values.mode_conditional = active;
}

void update_mode_variance(GtkWidget *w, gpointer data)
{

  gint active = 0;
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON 
        (radio_variance_in_range))) 
  { 
    active = 1;
  }
    
    
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON 
        (radio_variance_out_range))) 
  {
    active = 2;
  }

  mode_values.mode_variance = active;
}

void update_mode_advanced(GtkWidget *w, gpointer data)
{

  gint active = 0;
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON 
        (radio_advanced_in_range))) 
  { 
    active = 1;
  }
    
    
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON 
        (radio_advanced_out_range))) 
  {
    active = 2;
  }

  mode_values.mode_advanced = active;
}

void set_sensitivity_conditional(gboolean is_visiable)
{
    gtk_widget_set_sensitive (label_P1                   , is_visiable);
    gtk_widget_set_sensitive (entry_text_P1              , is_visiable);
    gtk_widget_set_sensitive (label_P2                   , is_visiable);
    gtk_widget_set_sensitive (entry_text_P2              , is_visiable);
    gtk_widget_set_sensitive (radio_conditional_in_range , is_visiable);
    gtk_widget_set_sensitive (radio_conditional_out_range, is_visiable);
}

void set_sensitivity_advanced(gboolean is_visiable)
{
    gtk_widget_set_sensitive (label_Q1                , is_visiable);
    gtk_widget_set_sensitive (entry_Q1                , is_visiable);
    gtk_widget_set_sensitive (label_Q2                , is_visiable);
    gtk_widget_set_sensitive (entry_Q2                , is_visiable);
    gtk_widget_set_sensitive (radio_advanced_in_range , is_visiable);
    gtk_widget_set_sensitive (radio_advanced_out_range, is_visiable);
}

void set_sensitivity_variance(gboolean is_visiable)
{
    gtk_widget_set_sensitive (label_sigma             , is_visiable);
    gtk_widget_set_sensitive (entry_text_sigma        , is_visiable);
    gtk_widget_set_sensitive (radio_variance_in_range , is_visiable);
    gtk_widget_set_sensitive (radio_variance_out_range, is_visiable);
}

void set_sensitivity_matrix_cond(gboolean is_visiable)
{
	guint8 size = matrix_trans.size;
	for (guint8 i = 0; i < size; i++)
	{
		for (guint8 j = 0; j < size; j++)
		{
      gtk_widget_set_sensitive (entry_matrix_cond[i][j], is_visiable);
    }
  }

    gtk_widget_set_sensitive (check_is_use_matrix_cond, is_visiable);
    gtk_widget_set_sensitive (label_matrix_cond       , is_visiable);
    gtk_widget_set_sensitive (label_divisor_cond      , is_visiable);
    gtk_widget_set_sensitive (label_shift_cond        , is_visiable);
    gtk_widget_set_sensitive (entry_divisor_cond      , is_visiable);
    gtk_widget_set_sensitive (entry_shift_cond        , is_visiable);
}


void update_mode(GtkWidget *w, gpointer data)
{
  gint active = 0;
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON 
        (radio_basic))) 
  { 
    active = 1;
    set_sensitivity_conditional(FALSE);
    set_sensitivity_advanced(FALSE);
    set_sensitivity_variance(FALSE);
    set_sensitivity_matrix_cond(FALSE);
  }
    
    
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON 
        (radio_conditional))) 
  {
    active = 2;
    set_sensitivity_conditional(TRUE);
    set_sensitivity_advanced(FALSE);
    set_sensitivity_variance(FALSE);
    set_sensitivity_matrix_cond(FALSE);
  }

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON 
        (radio_advanced))) 
  {
    active = 3;
    set_sensitivity_conditional(FALSE);
    set_sensitivity_advanced(TRUE);
    set_sensitivity_variance(FALSE);
    set_sensitivity_matrix_cond(TRUE);
  }
    
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON 
        (radio_variance))) 
  {
    active = 4;
    set_sensitivity_conditional(FALSE);
    set_sensitivity_advanced(FALSE);
    set_sensitivity_variance(TRUE);
    set_sensitivity_matrix_cond(TRUE);
  }

  mode_values.mode = active;
}


gboolean user_interface (const gchar *title, 
                         const gchar *help_id, 
                         GimpDrawable *drawable)
{
	GtkWidget *dialog;
	GtkWidget *main_hbox;
	GtkWidget *left_vbox;
	GtkWidget *left_mid_vbox;
 	GtkWidget *right_mid_vbox;
	GtkWidget *right_vbox;
	GtkWidget *label;

	gboolean   run;
	gimp_ui_init (PLUG_IN_BINARY, FALSE);
	dialog = gimp_dialog_new (title, PLUG_IN_ROLE,
                            NULL, 0,
                            gimp_standard_help_func, help_id,
                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                            GTK_STOCK_OK,     GTK_RESPONSE_OK,
                            NULL);

	//gimp_window_set_transient (GTK_WINDOW (dialog));
  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
	gtk_widget_show (dialog);

	//main_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
	main_hbox = gtk_hbox_new (0, 12);
	gtk_container_set_border_width (GTK_CONTAINER (main_hbox), 12);
	gtk_box_pack_start (
  GTK_BOX (gtk_dialog_get_content_area(GTK_DIALOG(dialog))), 
  main_hbox, TRUE, TRUE, 0);
	gtk_widget_show (main_hbox);

	//left_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
	left_vbox = gtk_vbox_new (0, 12);
	gtk_container_set_border_width (GTK_CONTAINER (left_vbox), 12);
	gtk_box_pack_start (GTK_BOX (main_hbox), left_vbox, TRUE, TRUE, 0);
	gtk_widget_show (left_vbox);

	//left_mid_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
	left_mid_vbox = gtk_vbox_new (0, 12);
	gtk_container_set_border_width (GTK_CONTAINER (left_mid_vbox), 12);
	gtk_box_pack_start (GTK_BOX (main_hbox), left_mid_vbox, TRUE, TRUE, 0);
	gtk_widget_show (left_mid_vbox);

	//right_mid_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
	right_mid_vbox = gtk_vbox_new (0, 12);
	gtk_container_set_border_width (GTK_CONTAINER (right_mid_vbox), 12);
	gtk_box_pack_start (GTK_BOX (main_hbox), right_mid_vbox, TRUE, TRUE, 0);
	gtk_widget_show (right_mid_vbox);

	//right_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
	right_vbox = gtk_vbox_new (0, 12);
	gtk_container_set_border_width (GTK_CONTAINER (right_vbox), 12);
	gtk_box_pack_start (GTK_BOX (main_hbox), right_vbox, TRUE, TRUE, 0);
	gtk_widget_show (right_vbox);

	// preview - left box
	label = gtk_label_new ("Podgląd");
	gtk_box_pack_start (GTK_BOX (left_vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);
 

  
  preview = gimp_drawable_preview_new (drawable, FALSE);
  preview_hbox = gimp_preview_get_controls ((GimpPreview *) preview);
  g_signal_connect_swapped (preview, "invalidated", 
      G_CALLBACK (update_preview), drawable);
  //preview_check = gtk_container_get_children(
  //    GTK_CONTAINER(preview_hbox))->data;
  
  // preview size
  gtk_widget_set_size_request (preview, 500, 500);
	gtk_box_pack_start (GTK_BOX (left_vbox), preview, FALSE, FALSE, 0);
  gtk_widget_show (preview);

	// Matrix - left middle box
	label = gtk_label_new ("Macierz transformacji");
	gtk_box_pack_start (GTK_BOX (left_mid_vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	// Create grid for matrix
	GtkWidget *table;	
	table = gtk_table_new(5, 5, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE (table), 2);
	gtk_table_set_col_spacings (GTK_TABLE (table), 4);

	guint8 size = matrix_trans.size;
  gchar c[10];
	for (guint8 i = 0; i < size; i++)
	{
		for (guint8 j = 0; j < size; j++)
		{
			entry_matrix_trans[i][j] = gtk_entry_new_with_max_length (4);
  
      sprintf(c, "%d", matrix_trans.matrix[i][j]);
			gtk_entry_set_text ((GtkEntry *)entry_matrix_trans[i][j], c);	
			gtk_entry_set_width_chars ((GtkEntry *)entry_matrix_trans[i][j], 4);		
			gtk_table_attach (GTK_TABLE (table), entry_matrix_trans[i][j], 
                        j, j+1, i, i+1, GTK_SHRINK, GTK_SHRINK, 0, 0);
			gtk_widget_show (entry_matrix_trans[i][j]);
		}
	}

	gtk_box_pack_start (GTK_BOX (left_mid_vbox), table, TRUE, TRUE, 0);
	gtk_widget_show (table);

	// under matrix
	GtkWidget *table_um;	
	table_um = gtk_table_new(2, 2, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE (table_um), 2);
	gtk_table_set_col_spacings (GTK_TABLE (table_um), 4);


  label = gtk_label_new ("Podzielnik: ");
  gtk_table_attach (GTK_TABLE (table_um), label, 0, 1, 0, 1, 
                    GTK_SHRINK, GTK_SHRINK, 0, 0);
  gtk_widget_show (label);

//	GtkWidget *divisor;
	entry_divisor_trans = gtk_entry_new_with_max_length (4);
  sprintf(c, "%d", matrix_trans.divider);
  gtk_entry_set_text ((GtkEntry *)entry_divisor_trans, c);	
	gtk_entry_set_width_chars ((GtkEntry *)entry_divisor_trans, 4);	
  gtk_table_attach (GTK_TABLE (table_um), entry_divisor_trans, 1, 2, 0, 1, 
                    GTK_SHRINK, GTK_SHRINK, 0, 0);
  gtk_widget_show (entry_divisor_trans);


	label = gtk_label_new ("Przesunięcie: ");
  gtk_table_attach (GTK_TABLE (table_um), label, 0, 1, 1, 2, 
                    GTK_SHRINK, GTK_SHRINK, 0, 0);
  gtk_widget_show (label);

	//GtkWidget *shift;
	entry_shift_trans = gtk_entry_new_with_max_length (4);
  sprintf(c, "%d", matrix_trans.shift);
  gtk_entry_set_text ((GtkEntry *)entry_shift_trans, c);	
	gtk_entry_set_width_chars ((GtkEntry *)entry_shift_trans, 4);	
  gtk_table_attach (GTK_TABLE (table_um), entry_shift_trans, 1, 2, 1, 2, 
                    GTK_SHRINK, GTK_SHRINK, 0, 0);
  gtk_widget_show (entry_shift_trans);

	gtk_box_pack_start (GTK_BOX (left_mid_vbox), table_um, TRUE, TRUE, 0);
	gtk_widget_show (table_um);

  /*
  // Break
  
  GtkWidget *frame_break = gtk_frame_new(NULL);
	gtk_box_pack_start (GTK_BOX (left_mid_vbox), frame_break, FALSE, FALSE, 0);
	gtk_widget_show (frame_break);
*/
  // conditional matrix *********************************** 
  check_is_use_matrix_cond = gtk_check_button_new_with_label 
    ("Uzyj macierzy warunkowej");
  gtk_toggle_button_set_active((GtkToggleButton *)check_is_use_matrix_cond, 
      TRUE);
  gtk_box_pack_start (GTK_BOX (left_mid_vbox), 
                      check_is_use_matrix_cond, FALSE, FALSE, 0);
  gtk_widget_set_sensitive (check_is_use_matrix_cond, FALSE);
  gtk_widget_show (check_is_use_matrix_cond);
  g_signal_connect (check_is_use_matrix_cond, "toggled", 
                    G_CALLBACK (matrix_conditional_sensitivity), NULL);

	label_matrix_cond = gtk_label_new ("Macierz warunkowa");
	gtk_box_pack_start (GTK_BOX (left_mid_vbox), label_matrix_cond, 
      FALSE, FALSE, 0);
  gtk_widget_set_sensitive (label_matrix_cond, FALSE);
	gtk_widget_show (label_matrix_cond);

	// Create grid for matrix
	GtkWidget *table_cond;	
	table_cond = gtk_table_new(5, 5, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE (table_cond), 2);
	gtk_table_set_col_spacings (GTK_TABLE (table_cond), 4);

	for (guint8 i = 0; i < size; i++)
	{
		for (guint8 j = 0; j < size; j++)
		{
			entry_matrix_cond[i][j] = gtk_entry_new_with_max_length (4);
  
      sprintf(c, "%d", matrix_cond.matrix[i][j]);
			gtk_entry_set_text ((GtkEntry *)entry_matrix_cond[i][j], c);	
			gtk_entry_set_width_chars ((GtkEntry *)entry_matrix_cond[i][j], 4);		
			gtk_table_attach (GTK_TABLE (table_cond), entry_matrix_cond[i][j], 
                        j, j+1, i, i+1,GTK_SHRINK, GTK_SHRINK, 0, 0);
      gtk_widget_set_sensitive (entry_matrix_cond[i][j], FALSE);
			gtk_widget_show (entry_matrix_cond[i][j]);
		}
	
  }


	gtk_box_pack_start (GTK_BOX (left_mid_vbox), table_cond, TRUE, TRUE, 0);
	gtk_widget_show (table_cond);

	// under matrix
	GtkWidget *table_cond_um;	
	table_cond_um = gtk_table_new(2, 2, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE (table_cond_um), 2);
	gtk_table_set_col_spacings (GTK_TABLE (table_cond_um), 4);


  label_divisor_cond = gtk_label_new ("Podzielnik: ");
  gtk_table_attach (GTK_TABLE (table_cond_um), label_divisor_cond, 
      0, 1, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
  gtk_widget_set_sensitive (label_divisor_cond, FALSE);
  gtk_widget_show (label_divisor_cond);

//	GtkWidget *divisor;
	entry_divisor_cond = gtk_entry_new_with_max_length (4);
  sprintf(c, "%d", matrix_cond.divider );
  gtk_entry_set_text ((GtkEntry *)entry_divisor_cond, c);	
	gtk_entry_set_width_chars ((GtkEntry *)entry_divisor_cond, 4);	
  gtk_table_attach (GTK_TABLE (table_cond_um), entry_divisor_cond,
      1, 2, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
  gtk_widget_set_sensitive (entry_divisor_cond, FALSE);
  gtk_widget_show (entry_divisor_cond);


	label_shift_cond = gtk_label_new ("Przesunięcie: ");
  gtk_table_attach (GTK_TABLE (table_cond_um), label_shift_cond,
      0, 1, 1, 2, GTK_SHRINK, GTK_SHRINK, 0, 0);
  gtk_widget_set_sensitive (label_shift_cond, FALSE);
  gtk_widget_show (label_shift_cond);

	//GtkWidget *shift;
	entry_shift_cond = gtk_entry_new_with_max_length (4);
  sprintf(c, "%d", matrix_cond.shift);
  gtk_entry_set_text ((GtkEntry *)entry_shift_cond, c);	
	gtk_entry_set_width_chars ((GtkEntry *)entry_shift_cond, 4);	
  gtk_table_attach (GTK_TABLE (table_cond_um), entry_shift_cond,
      1, 2, 1, 2, GTK_SHRINK, GTK_SHRINK, 0, 0);
  gtk_widget_set_sensitive (entry_shift_cond, FALSE);
  gtk_widget_show (entry_shift_cond);

	gtk_box_pack_start (GTK_BOX (left_mid_vbox), table_cond_um, TRUE, TRUE, 0);
	gtk_widget_show (table_cond_um);

  // Channels - right middle box
	label = gtk_label_new ("Kanały");
	gtk_box_pack_start (GTK_BOX (right_mid_vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

  if(draw_values.channels > 2)
  {
    //channel_red;
    channel_red = gtk_check_button_new_with_label ("Czerwony");
    gtk_toggle_button_set_active((GtkToggleButton *)channel_red, TRUE);
    gtk_box_pack_start (GTK_BOX (right_mid_vbox), 
                        channel_red, FALSE, FALSE, 0);
    gtk_widget_show (channel_red);

    //channel_green;
    channel_green = gtk_check_button_new_with_label ("Zielony");
    gtk_toggle_button_set_active((GtkToggleButton *)channel_green, TRUE);
    gtk_box_pack_start (GTK_BOX (right_mid_vbox), 
                        channel_green, FALSE, FALSE, 0);
    gtk_widget_show (channel_green);

    //channel_blue;
    channel_blue = gtk_check_button_new_with_label ("Niebieski");
    gtk_toggle_button_set_active((GtkToggleButton *)channel_blue, TRUE);
    gtk_box_pack_start (GTK_BOX (right_mid_vbox), 
                        channel_blue, FALSE, FALSE, 0);
    gtk_widget_show (channel_blue);
  }
  else
  {    
    //channel_grey;
    channel_grey = gtk_check_button_new_with_label ("Szary");
    gtk_toggle_button_set_active((GtkToggleButton *)channel_grey, TRUE);
    gtk_box_pack_start (GTK_BOX (right_mid_vbox), 
                        channel_grey, FALSE, FALSE, 0);
    gtk_widget_show (channel_grey);
  }

	//channel_alfa;
	channel_alfa = gtk_check_button_new_with_label ("Alfa");
    gtk_toggle_button_set_active((GtkToggleButton *)channel_alfa, FALSE);
	gtk_box_pack_start (GTK_BOX (right_mid_vbox), channel_alfa, FALSE, FALSE, 0);
	gtk_widget_show (channel_alfa);
  if(draw_values.channels == 2 || draw_values.channels == 4)
  { 
    gtk_widget_set_sensitive (channel_alfa, TRUE);
  } 
  else
  {
    gtk_widget_set_sensitive (channel_alfa, FALSE);
  }

	// Conditions - right box
	label = gtk_label_new ("Opcje / działanie: ");
	gtk_box_pack_start (GTK_BOX (right_vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

  GSList *group;
  GSList *group_neasted;
  GSList *group_neasted_v;
  GtkWidget *radio_button_neasted;

  radio_basic 	= gtk_radio_button_new_with_label (NULL, "Podstawowe");
	gtk_box_pack_start (GTK_BOX (right_vbox), 
											radio_basic, FALSE, FALSE, 0);
	gtk_widget_show (radio_basic);
  group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio_basic));



//****************************************************************

  radio_conditional = gtk_radio_button_new_with_label 
    (group, "Warunkowe");
	gtk_box_pack_start (GTK_BOX (right_vbox), 
											radio_conditional, FALSE, FALSE, 0);
	gtk_widget_show (radio_conditional);

  //    aaaaaaaaaaaaaaa
  GtkWidget *frame = gtk_frame_new(NULL);
 
  //GtkWidget *boxx = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
  GtkWidget *boxx = gtk_vbox_new (0, 0);
	//gtk_box_pack_start (GTK_BOX (frame), boxx, FALSE, FALSE, 0);
  gtk_container_add(GTK_CONTAINER (frame), boxx);
  gtk_container_set_border_width (GTK_CONTAINER (boxx), 5);
  gtk_widget_show (boxx);

	//GtkWidget *hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
	GtkWidget *hbox = gtk_hbox_new (0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
	gtk_box_pack_start (GTK_BOX (boxx), hbox, FALSE, FALSE, 0);
	gtk_widget_show (hbox);

  label_P1 = gtk_label_new ("P1 = ");
	gtk_box_pack_start (GTK_BOX (hbox), label_P1, FALSE, FALSE, 0);
	gtk_widget_show (label_P1);
  gtk_widget_set_sensitive (label_P1, FALSE);

  entry_text_P1 = gtk_entry_new_with_max_length (5);
  sprintf(c, "%d", mode_values.mode_conditional_left_value);
  gtk_entry_set_text ((GtkEntry *)entry_text_P1, c);	
	gtk_entry_set_width_chars ((GtkEntry *)entry_text_P1, 4);		
  gtk_box_pack_start (GTK_BOX (hbox), entry_text_P1, FALSE, FALSE, 0);
  gtk_widget_show (entry_text_P1);
  gtk_widget_set_sensitive (entry_text_P1, FALSE);

  label_P2 = gtk_label_new ("P2 = ");
	gtk_box_pack_start (GTK_BOX (hbox), label_P2, FALSE, FALSE, 0);
	gtk_widget_show (label_P2);
  gtk_widget_set_sensitive (label_P2, FALSE);

  entry_text_P2 = gtk_entry_new_with_max_length (5);
  sprintf(c, "%d", mode_values.mode_conditional_right_value);
  gtk_entry_set_text ((GtkEntry *)entry_text_P2, c);	
	gtk_entry_set_width_chars ((GtkEntry *)entry_text_P2, 4);		
  gtk_box_pack_start (GTK_BOX (hbox), entry_text_P2, FALSE, FALSE, 0);
  gtk_widget_show (entry_text_P2);
  gtk_widget_set_sensitive (entry_text_P2, FALSE);

  radio_conditional_in_range = 
    gtk_radio_button_new_with_label (NULL, "W zakresie <P1;P2>");

  if (mode_values.mode_conditional == 1)
  { 
  gtk_toggle_button_set_active (
      GTK_TOGGLE_BUTTON ((GtkToggleButton *)radio_conditional_in_range), TRUE);
  }
  gtk_box_pack_start (GTK_BOX (boxx), radio_conditional_in_range, FALSE, FALSE, 0);
  
  gtk_widget_show (radio_conditional_in_range);
  group_neasted = 
    gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio_conditional_in_range));
  gtk_widget_set_sensitive (radio_conditional_in_range, FALSE);

  radio_conditional_out_range = gtk_radio_button_new_with_label_from_widget (
      GTK_RADIO_BUTTON (radio_conditional_in_range), 
      "Poza zakresem <0;P1) ^ (P2;255>");
  if (mode_values.mode_conditional == 2)
  {
    gtk_toggle_button_set_active (
      GTK_TOGGLE_BUTTON ((GtkToggleButton *)radio_conditional_out_range), TRUE);
  }
  gtk_box_pack_start (GTK_BOX (boxx),  radio_conditional_out_range, FALSE, FALSE, 0);
  gtk_widget_show (radio_conditional_out_range);

  gtk_widget_set_sensitive (radio_conditional_out_range, FALSE);


  g_signal_connect (radio_conditional_in_range, "toggled", 
                    G_CALLBACK (update_mode_conditional), "1");
  g_signal_connect (radio_conditional_out_range, "toggled", 
                    G_CALLBACK (update_mode_conditional), "2");

  //gtk_container_add(GTK_CONTAINER(frame), boxx);
	gtk_box_pack_start (GTK_BOX (right_vbox), frame, FALSE, FALSE, 0);
	gtk_widget_show (frame);



//***********************************************************************
// Mode advanced
  radio_advanced = gtk_radio_button_new_with_label_from_widget (
      GTK_RADIO_BUTTON (radio_conditional), "Zaawansowane");
  gtk_box_pack_start (GTK_BOX (right_vbox), radio_advanced, 
      FALSE, FALSE, 0);
  gtk_widget_show (radio_advanced);

  //    aaaaaaaaaaaaaaa
  GtkWidget *frame_advanced = gtk_frame_new(NULL);
 
  // FIXME: in gtk frame is not box
  //GtkWidget *vbox_advanced = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
  GtkWidget *vbox_advanced = gtk_vbox_new (0, 0);
	//gtk_box_pack_start (GTK_BOX (frame_advanced), vbox_advanced, FALSE, FALSE, 0);
  gtk_container_add(GTK_CONTAINER (frame_advanced), vbox_advanced);
  gtk_container_set_border_width (GTK_CONTAINER (vbox_advanced), 5);
  gtk_widget_show (vbox_advanced);

//	GtkWidget *hbox_advanced = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
	GtkWidget *hbox_advanced = gtk_hbox_new (0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (hbox_advanced), 5);
	gtk_box_pack_start (GTK_BOX (vbox_advanced), hbox_advanced, FALSE, FALSE, 0);
	gtk_widget_show (hbox_advanced);

  label_Q1 = gtk_label_new ("Q1 = ");
	gtk_box_pack_start (GTK_BOX (hbox_advanced), label_Q1, FALSE, FALSE, 0);
	gtk_widget_show (label_Q1);
  gtk_widget_set_sensitive (label_Q1, FALSE);

  entry_Q1 = gtk_entry_new_with_max_length (5);
  sprintf(c, "%d", mode_values.mode_advanced_left_value);
  gtk_entry_set_text ((GtkEntry *)entry_Q1, c);	
	gtk_entry_set_width_chars ((GtkEntry *)entry_Q1, 4);		
  gtk_box_pack_start (GTK_BOX (hbox_advanced), entry_Q1, FALSE, FALSE, 0);
  gtk_widget_show (entry_Q1);
  gtk_widget_set_sensitive (entry_Q1, FALSE);

  label_Q2 = gtk_label_new ("Q2 = ");
	gtk_box_pack_start (GTK_BOX (hbox_advanced), label_Q2, FALSE, FALSE, 0);
	gtk_widget_show (label_Q2);
  gtk_widget_set_sensitive (label_Q2, FALSE);

  entry_Q2 = gtk_entry_new_with_max_length (5);
  sprintf(c, "%d", mode_values.mode_advanced_right_value);
  gtk_entry_set_text ((GtkEntry *)entry_Q2, c);	
	gtk_entry_set_width_chars ((GtkEntry *)entry_Q2, 4);		
  gtk_box_pack_start (GTK_BOX (hbox_advanced), entry_Q2, FALSE, FALSE, 0);
  gtk_widget_show (entry_Q2);
  gtk_widget_set_sensitive (entry_Q2, FALSE);

  radio_advanced_in_range = 
    gtk_radio_button_new_with_label (NULL, "Srednia wazona zakresie <Q1;Q2>");
  if (mode_values.mode_advanced == 1)
  {
  gtk_toggle_button_set_active (
      GTK_TOGGLE_BUTTON ((GtkToggleButton *)radio_advanced_in_range), TRUE);
  }

  gtk_box_pack_start (GTK_BOX (vbox_advanced),
    radio_advanced_in_range, FALSE, FALSE, 0);
  gtk_widget_show (radio_advanced_in_range);
  group_neasted = 
    gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio_advanced_in_range));
  gtk_widget_set_sensitive (radio_advanced_in_range, FALSE);


  radio_advanced_out_range = gtk_radio_button_new_with_label_from_widget (
      GTK_RADIO_BUTTON (radio_advanced_in_range), 
      "Srednia wazona poza zakresem <0;Q1) ^ (Q2;255>");
  if (mode_values.mode_advanced == 2)
  {
  gtk_toggle_button_set_active (
      GTK_TOGGLE_BUTTON ((GtkToggleButton *)radio_advanced_out_range), TRUE);
  }
  gtk_box_pack_start (GTK_BOX (vbox_advanced), radio_advanced_out_range,
      FALSE, FALSE, 0);
  gtk_widget_show (radio_advanced_out_range);

  gtk_widget_set_sensitive (radio_advanced_out_range, FALSE);


  g_signal_connect (radio_advanced_in_range, "toggled", 
                    G_CALLBACK (update_mode_advanced), "1");
  g_signal_connect (radio_advanced_out_range, "toggled", 
                    G_CALLBACK (update_mode_advanced), "2");

  //gtk_container_add(GTK_CONTAINER(frame_advanced), vbox_advanced);
	gtk_box_pack_start (GTK_BOX (right_vbox), frame_advanced, FALSE, FALSE, 0);
	gtk_widget_show (frame_advanced);






//***********************************************************************

  radio_variance = gtk_radio_button_new_with_label_from_widget (
      GTK_RADIO_BUTTON (radio_basic), "Wariancja");
  gtk_box_pack_start (GTK_BOX (right_vbox), radio_variance, 
      FALSE, FALSE, 0);
  gtk_widget_show (radio_variance);
 

  //    aaaaaaaaaaaaaaa
  GtkWidget *frame_v = gtk_frame_new(NULL);
 
  GtkWidget *box_v = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
	//gtk_box_pack_start (GTK_BOX (frame_v), box_v, FALSE, FALSE, 0);
  gtk_container_add(GTK_CONTAINER (frame_v), box_v);
  gtk_container_set_border_width (GTK_CONTAINER (box_v), 5);
  gtk_widget_show (box_v);

	GtkWidget *hbox_v;
	//hbox_v = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
	hbox_v = gtk_hbox_new (0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (hbox_v), 5);
	gtk_box_pack_start (GTK_BOX (box_v), hbox_v, FALSE, FALSE, 0);
	gtk_widget_show (hbox_v);

  // k parametr is equal to sigma
  //label_sigma = gtk_label_new ("Sigma = ");
  label_sigma = gtk_label_new ("k = ");
	gtk_box_pack_start (GTK_BOX (hbox_v), label_sigma, FALSE, FALSE, 0);
	gtk_widget_show (label_sigma);
  gtk_widget_set_sensitive (label_sigma, FALSE);

  entry_text_sigma = gtk_entry_new_with_max_length (5);
  sprintf(c, "%f", mode_values.mode_variance_sigma);
  gtk_entry_set_text ((GtkEntry *)entry_text_sigma, c);	
	gtk_entry_set_width_chars ((GtkEntry *)entry_text_sigma, 5);		
  gtk_box_pack_start (GTK_BOX (hbox_v), entry_text_sigma, FALSE, FALSE, 0);
  gtk_widget_show (entry_text_sigma);
  gtk_widget_set_sensitive (entry_text_sigma, FALSE);


  radio_variance_in_range = 
    gtk_radio_button_new_with_label (NULL, "W zakresie");
  if (mode_values.mode_variance == 1)
  {
  gtk_toggle_button_set_active (
      GTK_TOGGLE_BUTTON ((GtkToggleButton *)radio_variance_in_range), TRUE);
  }
	
  gtk_box_pack_start (GTK_BOX (box_v), radio_variance_in_range, FALSE, FALSE, 0);
	gtk_widget_show (radio_variance_in_range);
  group_neasted_v = 
    gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio_variance_in_range));
  gtk_widget_set_sensitive (radio_variance_in_range, FALSE);

  radio_variance_out_range = gtk_radio_button_new_with_label_from_widget (
      GTK_RADIO_BUTTON (radio_variance_in_range), 
      "Poza zakresem");
  if (mode_values.mode_variance == 2)
  {
  gtk_toggle_button_set_active (
      GTK_TOGGLE_BUTTON ((GtkToggleButton *)radio_variance_out_range), TRUE);
  }

  gtk_box_pack_start (GTK_BOX (box_v), radio_variance_out_range, FALSE, FALSE, 0);
  gtk_widget_show (radio_variance_out_range);

  gtk_widget_set_sensitive (radio_variance_out_range, FALSE);


  g_signal_connect (radio_variance_in_range, "toggled", 
                    G_CALLBACK (update_mode_variance), "1");
  g_signal_connect (radio_variance_out_range, "toggled", 
                    G_CALLBACK (update_mode_variance), "2");

  //gtk_container_add(GTK_CONTAINER(frame_v), box_v);
	gtk_box_pack_start (GTK_BOX (right_vbox), frame_v, FALSE, FALSE, 0);
	gtk_widget_show (frame_v);
//***********************************************************************

  g_signal_connect (radio_basic, "toggled", 
                    G_CALLBACK (update_mode), "1");
  g_signal_connect (radio_conditional, "toggled", 
                    G_CALLBACK (update_mode), "2");
  g_signal_connect (radio_advanced, "toggled", 
                    G_CALLBACK (update_mode), "2");
  g_signal_connect (radio_variance, "toggled", 
                    G_CALLBACK (update_mode), "3");
  

  gtk_toggle_button_set_active (
      GTK_TOGGLE_BUTTON (radio_basic), TRUE);
  gtk_toggle_button_set_active (
      GTK_TOGGLE_BUTTON (radio_conditional), FALSE);
  gtk_toggle_button_set_active (
      GTK_TOGGLE_BUTTON (radio_advanced), FALSE);
  gtk_toggle_button_set_active (
      GTK_TOGGLE_BUTTON (radio_variance), FALSE);

	GtkWidget *btn_start;
	btn_start = gtk_button_new_with_label ("Start");
	gtk_box_pack_start (GTK_BOX (right_vbox), btn_start, FALSE, FALSE, 0);
	gtk_widget_show (btn_start);
  
  g_signal_connect (btn_start, "clicked", 
                    G_CALLBACK (convolve_image), NULL);

	// End
	run = (gimp_dialog_run (GIMP_DIALOG (dialog)) == GTK_RESPONSE_OK);

	gtk_widget_destroy (dialog);

	return run;
}
