//#include "utils.h"
#include <ctype.h>
#include <stdlib.h>
#include <math.h>


//****************************************************
// structure for matrix
typedef struct
{
  guint radius;
  guint size;
  gint  matrix[5][5];

  guint size_index;
  guint tab_index[2][25];

  guint divider;
  guint shift;
} ConvolutionMatrix;

// matrix transformation
static ConvolutionMatrix matrix_trans = 
{
  2, // radius
  5, // size
  {
    {0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0},
    {0, 0, 1, 0, 0},
    {0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0},
  },
  1, // not zero values in matrix
  {
    {2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
  },
  1,
  0,
};

// matrix conditional
static ConvolutionMatrix matrix_cond = 
{
  2, // radius
  5, // size
  {
    {0, 0, 0,  0, 0},
    {0, 0, 0,  0, 0},
    {0, 0, 1,  0, 0},
    {0, 0, 0,  0, 0},
    {0, 0, 0,  0, 0},
  },
  1, // not zero values in matrix
  {
    {2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
  },
  1,
  0,
};

//*********************************************
// structure for save data
typedef struct
{
  gint matrix_trans[5][5];
  gint divider_trans;
  gint shift_trans;

  gint matrix_cond[5][5];
  gint divider_cond;
  gint shift_cond;
  
  gint P1;
  gint P2;
  gint conditonal_range;

  gint Q1;
  gint Q2;
  gint advanced_range;

  gfloat sigma;
  gint variance_range;

} DataToSave;

// saved data
static DataToSave data_to_save = {
  
  .matrix_trans = 
  {
    {0, 0, 0,  0, 0},
    {0, 0, 0, 0, 0},
    {0, 0, 1,  0, 0},
    {0, 0, 0,  0, 0},
    {0, 0, 0,  0, 0},
  },
  .divider_trans = 1,
  .shift_trans = 0,

  .matrix_cond =
  {
    {0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0},
    {0, 0, 1, 0, 0},
    {0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0},
  },
  .divider_cond = 1,
  .shift_cond = 0,
  
  .P1 = 0,
  .P2 = 255,
  .conditonal_range = 1,

  .Q1 = 0,
  .Q2 = 255,
  .advanced_range = 1,

  .sigma = 1.0,
  .variance_range = 1,
};


//***********************************************************
// structure for save data associteted with picture 
typedef struct 
{
  gint x1, x2, y1, y2;
  guint width, height;
  guint channels;
} DrawableValues;

static DrawableValues draw_values =
{
  0, 0, 0, 0,
  0, 0,
  0,
};

//*****************************************************************
// data for mode 
typedef struct
{
  guint channels[4];
  guint mode;   // Basic  = 1, Conditional = 2, Advanced = 3, variance = 4; 

  guint mode_conditional; // In range = 1, Out range = 2;
  gint mode_conditional_left_value;
  gint mode_conditional_right_value;

  guint mode_advanced; // In range = 1, Out range = 2;
  gint mode_advanced_left_value;
  gint mode_advanced_right_value;

  guint mode_variance; // In range = 1, out range = 2;
  gdouble mode_variance_sigma; 
} ModeValues;

static ModeValues mode_values = 
{
  {0, 0, 0, 0}, //channel
  1,   // mode

  1,   // mode_conditional
  0,   // mode_conditional_left_value
  255, // mode_conditional_right_value

  1,   // mode_advanced
  0,   // mode_advanced_left_value
  255, // mode_advanced_right_value

  1,   // mode__variance
  1.0,   // mode_variance_sigma
};

GimpPixelRgn draw_region;


int isnumber(const char *s)
{
  while (*s) 
  {
    if (isdigit(*s) == 0 && *s != '-') return 0;
    s++;
  }

  return 1;
}

int isnumber_float(const char *s)
{
  while (*s) 
  {
    int tmp = (*s == '.' || *s == ',' || *s == '-') ? 1 : 0;
    if (isdigit(*s) == 0 && tmp == 0) return 0;
    s++;
  }

  return 1;
}

gint get_number(GtkEntry *entry)
{
  const gchar *strr = gtk_entry_get_text(entry);
  gint valuee = 0;

  if (isnumber(strr)) valuee = atoi(strr);
  
  return valuee;
}

gfloat get_number_float(GtkEntry *entry)
{
  const gchar *strr = gtk_entry_get_text(entry);
  gfloat valuee = 0.0;

  if (isnumber_float(strr)) valuee = atof(strr);

  return valuee;
}


